
/****************************************************************************
 * HEADER ir_oid.h
 * 
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * PURPOSE:
 *    This header file contains the ASN.1 Object Identifiers used
 *    by this implementation of the IR Z39.50 version 2 1992 standard.
 *
 * SCOPE:
 *    All IR CSCs which reference object identifiers.
 *    
 * NOTES:
 *    The object identifiers defined in this header have been extracted
 *    from the ANSI Z39.50-1992 version 2 standard - appendix A.
 *
 * $Id: ir_oid.h,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: ir_oid.h,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 * 
 ****************************************************************************/

#ifndef IR_OID_H

#define IR_OID_H


/****************************************************************************
 * IR OID Constants: 
 ****************************************************************************/

/* Object Identifier classes known to this application. */
#define OID_CLASS_APPLICATION_CONTEXT        "1.2.840.10003.1"
#define OID_CLASS_ABSTRACT_SYNTAX        "1.2.840.10003.2"
#define OID_CLASS_ATTRIBUTE_SET            "1.2.840.10003.3"
#define OID_CLASS_DIAGNOSTIC_SET        "1.2.840.10003.4"
#define OID_CLASS_ABSTRACT_RECORD_SYNTAX    "1.2.840.10003.5"
#define OID_CLASS_XFER_SYNTAX_NON_BIB        "1.2.840.10003.6"
#define OID_CLASS_RESOURCE_REPORT_FORMAT    "1.2.840.10003.7.1000.35.1"
#define OID_CLASS_ACCESS_CONTROL    "1.2.840.10003.8.1000.35.1"
#define OID_CLASS_USER_INFO_FIELD        "1.2.840.10003.10"

/* Basic Z3950 Application Context OID */
#define OID_APP_CNTXT_BASIC_Z3950        "1.2.840.10003.1.1"

/* Abstract Syntax - Z3950 APDUs */
#define OID_ABSTRACT_SYNTAX_Z3950_APDUS        "1.2.840.10003.2.1"

/* Presentation Context negotiation abstract and transfer syntaxes */
#define OID_PRESENATION_ABS_XFER_SYNTAX        "1.1"

/* Attribute Set OID - Bib1 */
#define OID_ATTRIBUTE_SET_BIB1            "1.2.840.10003.3.1"

/* Diagnostic Set OID - Bib1 */
#define OID_DIAGNOSTIC_SET_BIB1            "1.2.840.10003.4.1"

/* Abstract Record Syntax OIDs - From Appendix E. */
#define OID_REC_SYN_UNIMARC            "1.2.840.10003.5.1"
#define OID_REC_SYN_INTERMARC            "1.2.840.10003.5.2"
#define OID_REC_SYN_CCF                "1.2.840.10003.5.3"
#define OID_REC_SYN_USMARC            "1.2.840.10003.5.10"
#define OID_REC_SYN_UKMARC            "1.2.840.10003.5.11"
#define OID_REC_SYN_NORMARC            "1.2.840.10003.5.12"
#define OID_REC_SYN_LIBRISMARC            "1.2.840.10003.5.13"
#define OID_REC_SYN_DANMARC            "1.2.840.10003.5.14"
#define OID_REC_SYN_FINMARC            "1.2.840.10003.5.15"
#define OID_REC_SYN_MAB                "1.2.840.10003.5.16"
#define OID_REC_SYN_CANMARC            "1.2.840.10003.5.17"
#define OID_REC_SYN_SBN                "1.2.840.10003.5.18"
#define OID_REC_SYN_PICAMARC            "1.2.840.10003.5.19"

/* Transfer Syntax for non-bibliographic records - There are none defined */

/* Resource Report Format OID - Bib1. */
#define OID_RESOURCE_REPORT_FMT_BIB1        "1.2.840.10003.7.1"

/* User Information Field */
#define OID_USER_INFO                "1.2.840.10003.10.1"
#endif
