/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tppresnt.c
 *
 * Purpose: This is the target process black box present result set program.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tppresnt.c,v 1.4 1994/03/29 00:30:36 irsuprt Exp irsuprt $
 * $Log: tppresnt.c,v $
 * Revision 1.4  1994/03/29  00:30:36  irsuprt
 * fix bug: correct correct # of parameters to tp_create_present_response()
 * for IR PROBLEM # 50
 *
 * Revision 1.3  1993/11/04  19:35:07  irsuprt
 * added #ifdef DEAN_WILDER_FORMAT to skip 24 bytes between leader and tag info
 * of Marc record. (PROBLEM 19.0)
 *
 * Revision 1.2  1993/11/03  16:04:02  irsuprt
 * free PRESENT_REFERENCE_ID (PROBLEM 20)
 *
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "tp.h"
#include "ir_oid.h"
#include "asn_incl.h"
#include "irerror.h"
#include "irconst.h"
#include "tpbbox.h"

/**************************** Extern Global Ids **************************/

extern int errno;

/*****************************************************************************
 *
 * Function: tp_fill_present_response
 *
 * Purpose: This function fills and creates the present response PDU. The
 *          DB record is filled with the data file 'canmarc'.
 *
 * Input Arguments:
 *          present_result -   0 = No error (DB record)
 *                             8 = Mandatory fields are invalid (Diag record)
 *                            31 = Resource control failed (Diag record)
 *                           104 = Access control failed (Diag record)
 *          PDU_preq - present request PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              PDU_presp - present response PDU pointer
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_fill_present_response(present_result, PDU_preq, PDU_presp)
    int present_result;
    APDU *PDU_preq;
    APDU *PDU_presp;
{

    NamePlusRecord **data_ptr;
    unsigned char temp_data[2000];
    int i, ch, max_rec_size;
    FILE *infile;
    char error_str[73];

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_fill_present_response\n");
#endif

    PDU_presp->a.presentResp = (struct PresentResp *) Asn1Alloc(sizeof(struct PresentResp));

    PDU_presp->choiceId = APDU_PRESENTRESP;

    PDU_presp->a.presentResp->referenceId.octetLen = PDU_preq->a.presentReq->referenceId.octetLen;
    PDU_presp->a.presentResp->referenceId.octs = (char *) Asn1Alloc(PDU_preq->a.presentReq->referenceId.octetLen);
    memcpy(PDU_presp->a.presentResp->referenceId.octs, PDU_preq->a.presentReq->referenceId.octs, PDU_preq->a.presentReq->referenceId.octetLen);

    PDU_presp->a.presentResp->numRecordsReturned = 1L;
    PDU_presp->a.presentResp->nextResultSetPosition = 0L;

    PDU_presp->a.presentResp->presentStatus = (AsnInt *) Asn1Alloc(sizeof(AsnInt));
    *PDU_presp->a.presentResp->presentStatus = 0;

    PDU_presp->a.presentResp->dbOrDiagnosticRec = (struct Records *) Asn1Alloc(sizeof(struct Records));

    if (present_result == 0)
    {

        PDU_presp->a.presentResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics = AsnListNew(sizeof(void*));
        data_ptr = AsnListAppend(PDU_presp->a.presentResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics);

        PDU_presp->a.presentResp->dbOrDiagnosticRec->choiceId = RECORDS_DBORSURDIAGNOSTICS;

        (*data_ptr) = Asn1Alloc(sizeof(NamePlusRecord));

        (*data_ptr)->databaseName.octetLen = 4;
        (*data_ptr)->databaseName.octs = (char *) Asn1Alloc(4);
        memcpy((*data_ptr)->databaseName.octs, "auls", 4);

        (*data_ptr)->namePlusRecordChoice = (NamePlusRecordChoice *) Asn1Alloc(sizeof(NamePlusRecordChoice));
        (*data_ptr)->namePlusRecordChoice->choiceId = NAMEPLUSRECORDCHOICE_DATABASERECORD;
        (*data_ptr)->namePlusRecordChoice->a.databaseRecord = (DatabaseRecord *) Asn1Alloc(sizeof(DatabaseRecord));

        ASN_MakeOid((char *) OID_REC_SYN_CANMARC, &(*data_ptr)->namePlusRecordChoice->a.databaseRecord->direct_reference);

        (*data_ptr)->namePlusRecordChoice->a.databaseRecord->indirect_reference = (AsnInt *) Asn1Alloc(sizeof(AsnInt));
        *(*data_ptr)->namePlusRecordChoice->a.databaseRecord->indirect_reference = 0;

        (*data_ptr)->namePlusRecordChoice->a.databaseRecord->data_value_descriptor.octetLen = 0;
        (*data_ptr)->namePlusRecordChoice->a.databaseRecord->data_value_descriptor.octs = NULL;

        (*data_ptr)->namePlusRecordChoice->a.databaseRecord->encoding = (struct EXTERNALChoice *) Asn1Alloc(sizeof(struct EXTERNALChoice));
        (*data_ptr)->namePlusRecordChoice->a.databaseRecord->encoding->choiceId = EXTERNALCHOICE_OCTET_ALIGNED;
        (*data_ptr)->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));

        if ((infile = fopen("canmarc", "rb")) == NULL)
        {
            printf("BBOX: tp_fill_present_response: canmarc not found\n");
            return (FAILED);
        }

        /* skip 50 bytes */
        for (i = 0; i < 50; i++)
            ch = fgetc(infile);

        /*
         * skip the first 2 records for they are filled in the present
         * response
         */
        i = 0;
        while ((ch = fgetc(infile)) != EOF)
        {
            if (ch == 0x1d)
                i++;
            if (i == 2)
                break;
        }

        /* skip 36 bytes */
        for (i = 0; i < 36; i++)
            ch = fgetc(infile);

#ifdef DEAN_WILDER_FORMAT
        /* skip next 24 bytes */
        for (i = 0; i < 24; i++)
            ch = fgetc(infile);
#endif

        i = 0;
        while ((ch = fgetc(infile)) != EOF)
            if (ch == 0x1d)
            {
                temp_data[i++] = ch;
                break;
            }
            else
                temp_data[i++] = ch;

        max_rec_size = i;

        (*data_ptr)->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octetLen = (unsigned long int) i;
        (*data_ptr)->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octs = (char *) Asn1Alloc(i);
        memcpy((*data_ptr)->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octs, temp_data, (*data_ptr)->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octetLen);

        fclose(infile);

        /* update the largest data record size */
        PDU_presp->a.presentResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->dataSize = max_rec_size;

    }
    else
    {

        *PDU_presp->a.presentResp->presentStatus = 5;

        /* create diagnostics record (error record) */
        PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics = (struct DiagRec *) Asn1Alloc(sizeof(struct DiagRec));

        PDU_presp->a.presentResp->dbOrDiagnosticRec->choiceId = RECORDS_NONSURDIAGNOSTICS;

        PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octetLen = strlen(OID_DIAGNOSTIC_SET_BIB1);
        PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octs = (char *) Asn1Alloc(PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octetLen);
        memcpy(PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octs, OID_DIAGNOSTIC_SET_BIB1, PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octetLen);

        PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->condition = present_result;        /* error code */

        switch (present_result)
        {
            case 8:
                strcpy(error_str, "THE MANDATORY FIELDS ARE INVALID");
                break;
            case 31:
                strcpy(error_str, "RESOURCE CONTROL FAILED");
                break;
            case 104:
                strcpy(error_str, "ACCESS CONTROL FAILED");
                break;
        }

        PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octetLen = strlen(error_str);
        PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octs = (char *) Asn1Alloc(PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octetLen);
        memcpy(PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octs, error_str, PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octetLen);
    }

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_create_present_response
 *
 * Purpose: This function creates the present response PDU, encodes, and
 *          creates the present response APDU for sending to IR kernel.
 *
 * Input Arguments:
 *          present_result -   0 = No error (DB record)
 *                             8 = Mandatory fields are invalid (Diag record)
 *                            31 = Resource control failed (Diag record)
 *                           104 = Access control failed (Diag record)
 *          PDU_presp - present request PDU
 *          data_ptr - temporary data buffer for encoding process
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              outputBuf - encoded APDU pointer
 *              APDU_length - the size of the encoded APDU
 *
 * External Variables:
 *
 * Function used:
 *              tp_fill_present_response()
 *              SBufInstallData()
 *              SBufResetInWriteRvsMode()
 *              BEncAPDU()
 *              ir_fwrite()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_create_present_response(present_result, PDU_preq, data_ptr, outputBuf, APDU_length)
    int present_result;
	APDU *PDU_preq;
    char **data_ptr;
    SBuf *outputBuf;
    int *APDU_length;
{
    APDU PDU_presp;
    int i;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX:Calling tp_create_present_response()\n");
#endif

    tp_fill_present_response(present_result, PDU_preq, &PDU_presp);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX: tp_create_present_response, begin dump data:\n");
    PrintPresentResp(DEBUG_FILE2, PDU_presp.a.presentResp, 0);
    fprintf(DEBUG_FILE2, "BBOX: tp_create_present_response, end dump data:\n");
#endif

    SBufInstallData(outputBuf, (*data_ptr), MAXMESSAGESIZE * 2);
    SBufResetInWriteRvsMode(outputBuf);

    *APDU_length = BEncAPDU(outputBuf, &PDU_presp);

	if (SBufWriteError(outputBuf))
	{
		display_error("KERNEL", ERR_ENCODE_APDU, -1);
		return(FAILED);
	}

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX:Present Response APDU Dump from tppresnt\n");
    ir_fwrite(SBufDataPtr(outputBuf), SBufDataLen(outputBuf), 1, DEBUG_FILE2);
#endif

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_process_present_request
 *
 * Purpose: This function performs the following functions:
 *              - checks the present request PDU,
 *              - performs the security checks,
 *              - performs the resource verification, and
 *              - creates and sends the present response APDU to IR kernel.
 *
 * Input Arguments:
 *              PDU_preq - present request PDU pointer
 *              data_ptr - temporary data pointer for ASN functions
 *              socketFd - the socket Fd for communicating to IR kernel
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              tp_check_present_request()
 *              tp_access_control()
 *              tp_resource_control()
 *              tp_create_present_response()
 *              tpa_send_APDU()
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_process_present_request(PDU_preq, data_ptr, socketFd)
    APDU *PDU_preq;
    char **data_ptr;
    int socketFd;
{
    int result;
    long APDU_length;
    SBuf APDU_presp;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_process_present_request\n");
#endif

	if (tp_access_control(socketFd, data_ptr, PDU_preq->a.presentReq->referenceId.octetLen, PDU_preq->a.presentReq->referenceId.octs) == SUCCESS)
	{
		if (tp_resource_control(socketFd, data_ptr, PDU_preq->a.presentReq->referenceId.octetLen, PDU_preq->a.presentReq->referenceId.octs) == FAILED)
		{
			display_error("BBOX", ERR_RESOURCE_CONTROL_FAILED, -1);
			tp_create_present_response(31, PDU_preq, data_ptr, &APDU_presp, &APDU_length);
		}
		else
			tp_create_present_response(0, PDU_preq, data_ptr, &APDU_presp, &APDU_length);
	}
	else
	{
		display_error("BBOX", ERR_ACCESS_CONTROL_FAILED, -1);
		tp_create_present_response(104, PDU_preq, data_ptr, &APDU_presp, &APDU_length);
	}

    if (tpa_send_APDU(socketFd, &APDU_presp, (int) APDU_length) <= 0)
    {
        display_error("TP_BBOX", ERR_SEND_DATA_TO_IR_FROM_TP, errno);
        return (FAILED);
    }
#ifdef DEBUG_LEVEL1
    else
        fprintf(DEBUG_FILE1, "BBOX: tp_process_present_request: %ld present response sent\n", APDU_length);
#endif

    return (SUCCESS);
}
